extends KinematicBody2D

# Everything for player movement
export var jump_speed = -1000
export var gravity = 50
export var friction = 0.3
export var acceleration = 150
var velocity = Vector2.ZERO
var enable_friction = false
var is_jumping = false
var snap = Vector2.ZERO

# Everything for wall jumping
export var wall_slide_acceleration = 10
export var max_wall_slide_speed = 120
var jump_dir = 0

# Everything for dashing
var move_dir = Vector2(0, 0)
var can_dash = true
var dashing = false
var dash_start_delta = 0
const COMBO_TIMEOUT = 0.5
var mouse_pos = Vector2.ZERO

# Self destroying tiles
onready var self_destroying_tiles = get_node("../self_destroying_tiles")
signal destruct_tile(pos, dir, use_jump_dir)

# Signal when player presses F
signal f_pressed()


# Basically handles jumping and falling
func movement_y():
	velocity.y += gravity
	if Input.is_action_just_pressed("jump"):
		if is_on_floor():
			is_jumping = true
			velocity.y = jump_speed
	
	if is_jumping && velocity.y >= 0:
		is_jumping = false
	
	velocity.y = clamp(velocity.y, -900, 700)



# Handles horizontal movement
func movement_x():
	# friction is defaulted to true, but if the player is moving
	# in the given frame, it is set to false
	enable_friction = true
	
	# If you are holding sprint, are on the floor, and are moving right
	# then go faster than usual, and turn off friction
	if Input.is_action_pressed("right") && Input.is_action_pressed("sprint") && is_on_floor():
		enable_friction = false
		velocity.x += acceleration
		velocity.x = clamp(velocity.x, -500, 500)
		
	# If you are moving right but not holding sprint, then
	# move like usual
	elif Input.is_action_pressed("right") && is_on_floor():
		enable_friction = false
		velocity.x += acceleration
		velocity.x = clamp(velocity.x, -150, 150)
		
	elif Input.is_action_pressed("right")  && is_on_floor() != true:
		enable_friction = false
		velocity.x += 30
		velocity.x = clamp(velocity.x, -400, 400)
		
	# If you are holding sprint, are on the floor, and are moving left
	# then go faster than usual, and turn off friction
	if Input.is_action_pressed("left") && Input.is_action_pressed("sprint") && is_on_floor():
		enable_friction = false
		velocity.x -= acceleration
		velocity.x = clamp(velocity.x, -500, 500)
		
	# If you are moving left but not holding sprint, then
	# move like usual
	elif Input.is_action_pressed("left") && is_on_floor():
		enable_friction = false
		velocity.x -= acceleration
		velocity.x = clamp(velocity.x, -150, 150)
		
	elif Input.is_action_pressed("left")  && is_on_floor() != true:
		enable_friction = false
		velocity.x -= 30
		velocity.x = clamp(velocity.x, -400, 400)
		
	# If friction is enabled, then apply friction
	if enable_friction == true:
		if is_on_floor():
			velocity.x = lerp(velocity.x, 0, friction)
		else:
			velocity.x = lerp(velocity.x, 0, 0.07)


# A function to handle wall jumps
func wall_jump():
	# This just sets the direction in which the raycasts should be facing
	if is_on_wall() && (Input.is_action_pressed("right")):
		jump_dir = 1
	
	elif is_on_wall() && (Input.is_action_pressed("left")):
		jump_dir = -1
		
	# Casts rays based on "dir"
	$RayCast2D.set_cast_to(Vector2(jump_dir*35, 0))
	$RayCast2D2.set_cast_to(Vector2(jump_dir*60, 0))
	
	# If player is next to a wall, then double jump.
	if $RayCast2D2.get_collider():
			if Input.is_action_just_pressed("jump") && $jump_timer.is_stopped():
				velocity.y = -800
				velocity.x = (jump_dir * -1) * 250
				$jump_timer.start()
				
			elif Input.is_action_pressed("grapple") && $RayCast2D.get_collider():
				velocity.y = 0
				
			elif $RayCast2D.get_collider() && velocity.y > 0:
				velocity.y = min(velocity.y + wall_slide_acceleration, max_wall_slide_speed)


func dash(delta):
#	if Input.is_action_pressed("dash_left"):
#		move_dir.x = -1
#
#	elif Input.is_action_pressed("dash_right"):
#		move_dir.x = 1
#
#	else:
#		move_dir.x = 0
#
#	if Input.is_action_pressed("up"):
#		move_dir.y = -1
#
#	else:
#		move_dir.y = 0
	if Input.is_action_just_pressed("left_mouse_button") && can_dash == true:
		mouse_pos = get_local_mouse_position().normalized()
		if mouse_pos.y > 0 && is_on_floor():
			mouse_pos.y = 0
		dashing = true
		can_dash = false
		
	# If you can dash, and you want to dash, then dash
#	if Input.is_action_just_pressed("shift") && can_dash == true:
#		if last_key_delta < COMBO_TIMEOUT:
#			dashing = true
#			can_dash = false
#		last_key_delta = 0 

	if dashing == true:
		# Because timers are unreliable at small amounts of time,
		# I used delta to keep track of time, instead of a timer.
		if dash_start_delta < 0.06:
			dash_start_delta += delta
			velocity = mouse_pos.normalized() * 1500
			enable_friction = false # For the time you are dashing, turn off friction
		else:
			dashing = false
			dash_start_delta = 0


# Checks if 
func self_destroying_tiles_check():
	if $tile_check_ray.is_colliding():
		emit_signal("destruct_tile", $tile_check_ray.get_collision_point(), jump_dir, false)
		
	if $tile_check_ray2.is_colliding():
		emit_signal("destruct_tile", $tile_check_ray2.get_collision_point(), jump_dir, false)
		
	if $RayCast2D2.is_colliding() != false:
		if $RayCast2D2.get_collider().is_in_group("self_destructing_walls"):
			emit_signal("destruct_tile", $RayCast2D2.get_collision_point(), jump_dir, true)


func _physics_process(delta):
#	print(get_global_mouse_position())
	if GlobalVariables.in_control_of_player == true:
		movement_x()
		movement_y()
		wall_jump()
		dash(delta)
		self_destroying_tiles_check()
		
		if is_jumping == false:
			snap = Vector2.DOWN * 32
		else:
			snap = Vector2.ZERO
			
		velocity = move_and_slide_with_snap(velocity, snap, Vector2.UP)
		
		if Input.is_action_just_pressed("interact"):
			emit_signal("f_pressed")


# Every time this timer stops, the can_dash variable is updated
func _on_update_can_dash_timeout():
	if is_on_floor():
		can_dash = true
